#! /usr/bin/env python
# -*- coding: iso-8859-15 -*-

""" Script de lancement de mumudvb

Auteur : Frdric Pauget
Licence : GPLv2
"""
import sys, getopt, os, time, copy, signal
from dvb_base import NotRunning, CarteOqp

import dvb_conf

sys.path.append('/usr/scripts/python-lib')
import lock
from daemon import daemonize

LOCK='/var/run/mumudvb/tv'

def usage(erreur=None) :
    if erreur : print erreur
    print """Usage : 
  %(p)s start [<numero carte> [<transpondeur>]]: 
      dmarrage le transpondeur donn sur la carte donne, 
      si le transpondeur est omis dmarre celui dfini dans la conf
      si seul dmarre des flux dfinis dans la conf
  %(p)s stop [numero carte] : 
      arrte tous les flux des cartes spcifies, 
      si aucune carte est fournie arrte tous les flux
      
Les options possibles sont :
    -d ou --debug : affiche tous les messages 
                    et ne daemonize pas crans_dvbsream
    -v ou --verbose : affiche les messages de debuggage
    -q ou --quiet : affiche rien
    --timeout_accord=<nb> : nb de secondes donnes pour l'accord""" \
    % { 'p' : sys.argv[0].split('/')[-1] + ' [options]'}
    if not erreur : sys.exit(0)
    else : sys.exit(-1)
    
def analyse_opt() :
    """ Traitement des options 
    Retourne le niveau de verbosit et les arguments """
    # Options par dfaut
    verbose = 1
    timeout_accord = 20

    try :
        options, args = getopt.getopt(sys.argv[1:], 'hdvq', [ 'help', 'debug' , 'quiet' , 'verbose', 'timeout_accord='] )
    except getopt.error, msg :
        usage('%s\n' % msg)
    
    for opt, val in options :
        if opt in [ '-v' , '--verbose' ] :
            verbose = 2
        elif opt in [ '-d' , '--debug' ] :
            verbose = 3
        elif opt == [ '-q' , '--quiet' ] :
            verbose = 0
        elif opt == '--timeout_accord' :
            try:
                timeout_accord = int(val)
            except ValueError:
                usage("Valeur de timeout_accord (%s) incorrecte" % val)
        elif opt in [ '-h', '--help' ] :
            usage()

    return verbose, timeout_accord, args

def term(a=None,b=None) :
    """ Tue tous les mumudvb et quitte """
    stop(range(6))
    lock.remove_lock(LOCK)
    sys.exit(0)
    
def stop(liste) :
    """ Coupe la diffusion des cartes dont le numro est dans la liste """
    from dvb_base import carte
    for i in liste :
        print "stop %i" % carte(i).card
        #carte(i).stop()

def hup(sig,frame) :
    """ Reli la conf et stop les cartes dont la conf  chang """
    orig_conf = frame.f_globals['conf']
    conf = config(frame.f_globals['verbose'],
                  frame.f_globals['timeout_accord'])
                  
    # Quelles cartes faut-il-arrter ?
    to_stop=range(6) # carte pouvant tre arrtes
    for carte in orig_conf :
        print "Test carte %i (%s)" % (carte.card, carte.freq)
        if carte in conf :
            try:
                to_stop.remove(carte.card)
            except ValueError:
                # N'tait pas dans la liste
                pass
            
    stop(to_stop)
    frame.f_globals['conf']=conf
    
def config(verbose,timeout_accord) :
    """ Retourne une liste d'instance de carte """
    reload(dvb_conf)
    
    # Config des cartes
    for carte in dvb_conf.conf :
        carte.verbose = verbose
        carte.timeout_accord = timeout_accord
        
    return dvb_conf.conf
        
if __name__ == '__main__' :
    ## Lancement par l'utilisateur tv
    if os.getuid() == 0 :
        os.system('su tv --command="%s"' % ' '.join(sys.argv))
        sys.exit(0)
    elif os.getuid() != 101 :
        print "Ce programme doit tre lanc par l'utilisateur tv (uid=101)"
        print "Astuce : sudo -u tv %s" % sys.argv[0]
        sys.exit(1)
        
    verbose, timeout_accord , args = analyse_opt()

    if verbose < 2 : 
        daemonize()
        
    # Lock
    lock.make_lock(LOCK,"Lancement des mumudvb")
    
    # Signal handler
    signal.signal(signal.SIGTERM,term)
    signal.signal(signal.SIGINT,term)
    signal.signal(signal.SIGHUP,hup)
        
    conf = config(verbose,timeout_accord)
        
    # Lancement de la TV        
    while 1 :
        for carte in conf :
            try :
                print "Carte %i" %  carte.card
#               carte.start()
            except CarteOqp :
                print "Carte %i occupe, abandon" % carte.card
            except NotRunning :
                # On retentera plus tard
                pass
        time.sleep(60)

